@@ -1,4 +1,4 @@
-import os, os.path as osp, logging, pprint, uuid
+import os, os.path as osp, logging, pprint, uuid, re
 from contextlib import contextmanager
 from collections import OrderedDict
 
@@ -166,6 +166,7 @@ def open_root(rootfile):
         'JetsAK15_ecfN2b1', 'JetsAK15_ecfN2b2',
         'JetsAK15_girth', 'JetsAK15_ptD',
         'JetsAK15_axismajor', 'JetsAK15_axisminor',
+        'HT',
         'MET', 'METPhi',
         'TriggerPass',
         'NMuons', 'NElectrons',
@@ -356,6 +357,15 @@ def filter_stitch(array):
     return copy
 
 
+def filter_at_least_one_ak8jet(array):
+    copy = array.copy()
+    a = copy.array
+    a = a[ak.count(a['JetsAK8.fCoordinates.fPt'], axis=-1)>=1] # At least one jet
+    copy.cutflow['>=1jets'] = len(a)
+    copy.array = a
+    return copy
+
+
 class Columns:
     """
     Class that contains a dictionary of arrays. Each array is guaranteed
@@ -547,3 +557,98 @@ def bdt_feature_columns(array):
 
     cols.arrays = a
     return cols
+
+
+def triggerstudy_columns(array):
+    a = array.array # Just to avoid typing array.array everywhere
+
+    # Most triggers are not interesting for us and they take up space
+    # Select all the triggers we would like to keep for further study
+    trigger_names = array.trigger_branch
+    trigger_set = set(triggers_2016 + triggers_2018)
+    keep_trigger_mask = np.array([(t in trigger_set) for t in trigger_names])
+
+    cols = Columns()
+    cols.cutflow = array.cutflow
+    cols.metadata = array.metadata
+    cols.metadata['trigger_titles'] = [t for t in array.trigger_branch if t in trigger_set]
+
+    # AK8 jets
+    pt_ak8 = a['JetsAK8.fCoordinates.fPt']
+    njets = ak.count(pt_ak8, axis=-1).to_numpy()
+    cols.arrays['njets'] = njets
+    cols.arrays['pt'] = pt_ak8[:,0].to_numpy()
+    cols.arrays['pt_subl'] = np.ones_like(cols.arrays['pt']) * np.nan
+    if np.any(njets>=2): cols.arrays['pt_subl'][njets>=2] = pt_ak8[njets>=2,1].to_numpy()
+
+    # AK15 jets
+    pt_ak15 = a['JetsAK15.fCoordinates.fPt']
+    njets_ak15 = ak.count(pt_ak15, axis=-1)
+    cols.arrays['njets_ak15'] = njets_ak15
+    cols.arrays['pt_ak15'] = np.ones_like(cols.arrays['pt']) * np.nan
+    if np.any(njets_ak15>=1): cols.arrays['pt_ak15'][njets_ak15>=1] = a['JetsAK15.fCoordinates.fPt'][njets_ak15>=1,0].to_numpy()
+    cols.arrays['pt_ak15_subl'] = np.ones_like(cols.arrays['pt']) * np.nan
+    if np.any(njets_ak15>=2): cols.arrays['pt_ak15_subl'][njets_ak15>=2] = a['JetsAK15.fCoordinates.fPt'][njets_ak15>=2,1].to_numpy()
+
+    # Event-level variables
+    cols.arrays['ht'] = a['HT'].to_numpy()
+    cols.arrays['met'] = a['MET'].to_numpy()
+    cols.arrays['weight'] = a['Weight'].to_numpy()
+
+    cols.arrays['triggers'] = a['TriggerPass'][:, keep_trigger_mask].to_numpy()
+    assert cols.arrays['triggers'].shape[1] == len(cols.metadata['trigger_titles'])
+    return cols
+
+
+def metadata_from_filename(path):
+    """
+    Heuristic to extract physics parameters from a path and stores it in a metadata dict.
+    
+    """
+    metadata = {}
+    # SIGNAL
+    match = re.search(r'year(\d+)', path)
+    metadata['year'] = int(match.group(1)) if match else 2018
+    match = re.search(r'genjetpt(\d+)', path)
+    if match: metadata['genjetpt'] = int(match.group(1))
+    match = re.search(r'madpt(\d+)', path)
+    if match: metadata['madpt'] = int(match.group(1))
+    match = re.search(r'mz(\d+)', path)
+    if match: metadata['mz'] = int(match.group(1))
+    match = re.search(r'mdark(\d+)', path)
+    if match: metadata['mdark'] = int(match.group(1))
+    match = re.search(r'rinv(\d\.\d+)', path)
+    if match: metadata['rinv'] = float(match.group(1))
+    # BACKGROUND
+    for bkg in ['qcd', 'ttjets', 'zjets', 'wjets']:
+        if bkg in path.lower():
+            metadata['bkg'] = bkg
+            logger.debug('Determined bkg=%s', bkg)
+            if bkg == 'ttjets':
+                for ch in ['SingleLeptFromTbar', 'SingleLeptFromT', 'DiLept']:
+                    if ch in path:
+                        metadata['channel'] = ch.lower()
+                        break
+                else:
+                    metadata['channel'] = 'inclusive'
+                logger.debug('Determined channel=%s', metadata['channel'])
+            break
+    match = re.search(r'20ul(\d+)', path.lower())
+    if match: metadata['year'] = 2000 + int(match.group(1))
+    match = re.search(r'HT\-(\d+)[tT]o([\dInf]+)', path)
+    if match:
+        left = int(match.group(1))
+        right = np.inf if match.group(2) == 'Inf' else int(match.group(2))
+        metadata['ht'] = (left, right)
+        logger.debug(f'Setting ht=({metadata["ht"]})')
+    match = re.search(r'Pt_(\d+)to([\dInf]+)', path)
+    if match:
+        left = int(match.group(1))
+        right = np.inf if match.group(2) == 'Inf' else int(match.group(2))
+        metadata['pt'] = (left, right)
+        logger.debug(f'Setting pt=({metadata["pt"]})')
+    match = re.search(r'genMET\-(\d+)', path)
+    if match:
+        metadata['genmet'] = int(match.group(1))
+        logger.debug('Setting genmet=%s', metadata['genmet'])
+    return metadata
